#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3				// Use modern global access method and strict wave access
#pragma DefaultTab={3,20,4}		// Set default tab width in Igor Pro 9 and later

// create a sample netCDF-4 or netCDF-4 classic model, test the output using e.g. nccdump 

// netCDF("myNC4",0) - make a netCDF-4 enhanced metadata file using extended data types and groups
// netCDF("myNC4classic",1) - make a netCDF-4 classic model files using same inputs as for netCDF-4
// same as calling nccopy -7 myNC4.nc myNC4classic.nc

// ncdump -k myFile.nc -> 'classic','netCDF-4 classic model','netCDF-4' 

// ncdump -h -s myFile.nc -> more information and more thorough than -k

// nccopy -3 myNC4classic.nc myNC3.nc

// ExecuteScriptText/B/Z "cmd.exe /C nccopy -3 myNC4classic.nc myNC3.nc 
//___________________________________________________________________________________________________	
// create a sample netCDF to test the writing functions
Function netCDF(String outfNameStr,int classic)
	
	double fileID
	
	// make some waves to output
	Make/O/I xWave=x // unlimited co-ordinate variable
	Make/O/N=(128,2) yWave=gnoise(x) // dataset
	Make/O/N=2/B zWave=x // hidden co-ordinate 
	Make/O/N=(128,2,2) aWave=gnoise(x^2) // dataset
	Make/O/N=2/B/U bWave=x+1 // co-ordinate variable
	Make/O/T/N=2 dWave={"left","right"} 
	Make/O/N=64/B cWave=x
	
	// get a path to make the netCDF file in
	NewPath/O/Q/Z/M= "Please select a folder to output a file to" dataFilePath 
	if (V_flag !=0)
		return -1
	endif
	
	// create the netCDF file
	fileID = QLA_create("dataFilePath",outfNameStr,classic,1)

	// output global attributes
	QLA_put_glob_att_text("/","Conventions","CF-1.8",fileID)
	QLA_put_glob_att_text("/","title","This is my Igor netCDF",fileID)
	QLA_put_glob_att_text("/","history",date()+", "+time(),fileID)
	QLA_put_glob_att_text("/","institution","WaveMetrics",fileID)
	QLA_put_glob_att_text("/","source","Igor Pro",fileID)
	QLA_put_glob_att_text("/","comment","This netCDF was made with the QLA netCDF-4 library",fileID)
	QLA_put_glob_att_text("/","references","https://www.unidata.ucar.edu/software/netcdf/docs/netcdf_introduction.html",fileID)

	// save xWave as unlimited dimension variable 
	QLA_def_unlim_dim_var(xWave,"/xWave",0,0,0,fileID)
	QLA_put_att_text("/xWave","long_name","this is the X dimension",fileID)
	QLA_put_att_text("/xWave","standard_name","some_fictitious_standard_name",fileID)
	QLA_put_att_text("/xWave","units","1",fileID)
	QLA_put_att_text("/xWave","comment","this is the X dimension",fileID)

	// save zWave as a dimension
	QLA_def_dim(zWave,"/zWave",0,0,0,fileID)
	
	// save text wave 
	QLA_def_var(dWave,"/dWave","/zWave;",0,0,0,fileID,storage=1)
	QLA_put_att_text("/dWave","long_name","string variable",fileID)
	QLA_put_att_text("/dWave","standard_name","some_fictitious_standard_name",fileID)
	QLA_put_att_text("/dWave","units","1",fileID)
	QLA_put_att_text("/dWave","comment","this is the X dimension",fileID)

	// save yWave variable, add a comment attribute
	QLA_def_var(yWave,"/yWave","/xWave;/zWave;",0,9,1,fileID)
	QLA_put_att("/yWave","variable",1,192,fileID)
	QLA_put_att_text("/yWave","comment","this is a data variable",fileID)

	// great a group in the file
	QLA_def_grp("/sub-group",fileID)
 	
	// output group attributes
	QLA_put_glob_att_text("/sub-group/","title","This is a netCDF-4 group",fileID)
	
	// save bWave as a dimension variable
	QLA_def_dim_var(bWave,"/sub-group/bWave",0,0,0, fileID)

	// save aWave to the group, add a comment attribute 
	QLA_def_var(aWave,"/sub-group/aWave","/xWave;/zWave;/sub-group/bWave;",0,0,0,fileID)
	QLA_put_att_text("/sub-group/aWave","comment","this another data variable",fileID)
	QLA_put_att("/sub-group/aWave","frequency",1,32,fileID)
	QLA_put_att_text("/sub-group/aWave","units","hippos",fileID)
	QLA_put_att("/sub-group/aWave","_FillValue",1,8,fileID)

	// do something to yWave that means it needs updating on disk
	yWave/=2
	// update the data in yWave on disk preserving the attributes
	QLA_update_var(yWave,"/yWave",fileID)
	
	
	// write some data in two halves
	QLA_def_var(cWave,"/cWave","/xWave;",0,0,0,fileID,extend=0)
	Reverse cWave
	QLA_def_var(cWave,"/cWave","/xWave;",0,0,0,fileID,extend=0)
	
	// close the file
	QLA_close(fileID)
	
	// clean up
	KillWaves/Z xWave,yWave,zWave,aWave,bWave,cWave,dWave
end
//___________________________________________________________________________________________________	
